%% This file is a colorplane converter between RGB and Ycbcr

% YCbCr Color Space
% The YCbCr color space is widely used for digital video. 
% In this format, luminance information is stored as a single component (Y), 
% and chrominance information is stored as two color-difference components 
% (Cb and Cr). Cb represents the difference between the blue component and 
% a reference value. Cr represents the difference between the red component 
% and a reference value. 
% (YUV, another color space widely used for digital video, is very similar 
% to YCbCr but not identical.)
% 
% YCbCr data can be double precision, but the color space is particularly 
% well suited to uint8 data. For uint8 images, the data range for 
% Y is [16, 235], and the range for Cb and Cr is [16, 240]. 
% YCbCr leaves room at the top and bottom of the full uint8 range so 
% that additional (nonimage) information can be included in a video stream.
% 
% The function rgb2ycbcr converts colormaps or RGB images to the 
% YCbCr color space. ycbcr2rgb performs the reverse operation.
% 
% For example, these commands convert an RGB image to YCbCr format.

clear all;
close all;

% RGB data [..,..,1]-Red data, [..,..,2]-Green data, [..,..,3]-Blue data
RGB = imread('altera_de2.jpg');
YCBCR = rgb2ycbcr(RGB);

[row_size,column_size,depth] = size (RGB);

figure(1);
subplot(2,2,1),image(ycbcr2rgb(YCBCR)),title('RGB version');
subplot(2,2,2),imshow(YCBCR(:,:,1)),title('Y channel');
subplot(2,2,3),imshow(YCBCR(:,:,2)),title('Cb Channel');
subplot(2,2,4),imshow(YCBCR(:,:,3)),title('Cr Channel');

figure(2);
subplot(2,2,1),image(RGB),title('RGB version');
subplot(2,2,2),imshow(RGB(:,:,1)),title('Red channel');
subplot(2,2,3),imshow(RGB(:,:,2)),title('Green Channel');
subplot(2,2,4),imshow(RGB(:,:,3)),title('Blue Channel');

%%

%%Inverting colors
for i= 1:1:row_size,
    for n = 1:1:column_size,
        RGB(i,n,1) = 255 - RGB(i,n,1);
        RGB(i,n,2) = 255 - RGB(i,n,2);
        RGB(i,n,3) = 255 - RGB(i,n,3);
    end
end

figure(3);
subplot(2,2,1),image(RGB),title('Inverted RGB version');
subplot(2,2,2),imshow(RGB(:,:,1)),title('Red channel');
subplot(2,2,3),imshow(RGB(:,:,2)),title('Green Channel');
subplot(2,2,4),imshow(RGB(:,:,3)),title('Blue Channel');

%%

brightness_factor = -50;
YCBCR_brightness = uint8(zeros(row_size,column_size,depth));

%%Adjusting brightness
for i= 1:1:row_size,
    for n = 1:1:column_size,
        YCBCR_brightness(i,n,1) = YCBCR(i,n,1)+brightness_factor;
        YCBCR_brightness(i,n,2) = YCBCR(i,n,2);
        YCBCR_brightness(i,n,3) = YCBCR(i,n,3);
    end
end

%This picture shows an adjustment of the Luma/Y component
figure(4);
subplot(2,2,1),image(ycbcr2rgb(YCBCR_brightness)),title('RGB version');
subplot(2,2,2),imshow(YCBCR_brightness(:,:,1)),title('Y channel');
subplot(2,2,3),imshow(YCBCR_brightness(:,:,2)),title('Cb Channel');
subplot(2,2,4),imshow(YCBCR_brightness(:,:,3)),title('Cr Channel');

%%

%Black and white picture
RBG_grayscale = uint8(zeros(row_size,column_size,depth));

for i= 1:1:row_size,
    for n = 1:1:column_size,
        RBG_grayscale(i,n,1) = YCBCR(i,n,1);
        RBG_grayscale(i,n,2) = YCBCR(i,n,1);
        RBG_grayscale(i,n,3) = YCBCR(i,n,1);
    end
end

figure(5);
imshow(RBG_grayscale);

%%

%Adjusting contrast

YCBCR_contrast = uint8(zeros(row_size,column_size,depth));
contrast_factor = 0.9;

for i= 1:1:row_size,
    for n = 1:1:column_size,
        YCBCR_contrast(i,n,1) = YCBCR(i,n,1);
        YCBCR_contrast(i,n,2) = YCBCR(i,n,2);
        YCBCR_contrast(i,n,3) = YCBCR(i,n,3);
    end
end

YCBCR_contrast = uint8(uint8(YCBCR_contrast)*contrast_factor);

figure(6);
subplot(2,2,1),image(ycbcr2rgb(YCBCR_contrast)),title('RGB version');
subplot(2,2,2),imshow(YCBCR_contrast(:,:,1)),title('Y channel');
subplot(2,2,3),imshow(YCBCR_contrast(:,:,2)),title('Cb Channel');
subplot(2,2,4),imshow(YCBCR_contrast(:,:,3)),title('Cr Channel');


%% The transform between the two domains

RGB_data = [13;100;120];%[Red;Green;Blue]
rbgTOycbcr = [16;128;128] + [0.257,0.504,0.098;-0.148,-0.291,0.439;0.439,-0.368,-0.071] * RGB_data;
ycbcrTOrgb = uint8([1.164,0.000,1.596;1.164,-0.392,-0.813;1.164,2.017,0.000] * [rbgTOycbcr(1) - 16;rbgTOycbcr(2) - 128;rbgTOycbcr(3) - 128]);

RGB_data
ycbcrTOrgb




%%